@@ -55,6 +55,8 @@ function EAction(guiAction) {
     // set to true to prefer dialog over options tool bar:
     this.useDialog = false;
     this.dialogUiFile = undefined;
+
+    this.waitingForContextMenu = false;
 }
 
 EAction.prototype = new RActionAdapter();
@@ -158,6 +160,8 @@ EAction.prototype.finishEvent = function() {
     if (!isNull(di)) {
         di.repaintViews();
     }
+
+    this.setCommandPrompt();
 };
 
 /**
@@ -701,15 +705,19 @@ EAction.prototype.keyPressEvent = function(event) {
         return;
     }
 
-    if ((event.key() >= Qt.Key_0.valueOf() && event.key() <= Qt.Key_9.valueOf()) || event.key() === Qt.Key_Period.valueOf()) {
-        // number or dot entered:
-        // give focus to first input widget in options tool bar and set text to number entered:
-        var w = OptionsToolBar.getFirstInputWidget();
-        if (!isNull(w)) {
-            w.setFocus(Qt.OtherFocusReason);
-            w.text = String.fromCharCode(event.key());
-            event.accept();
-            return;
+    // prevent focus shift when entering keycode (e.g. "D2"):
+    var appWin = EAction.getMainWindow();
+    if (appWin.getKeyLog().length===0) {
+        if ((event.key() >= Qt.Key_0.valueOf() && event.key() <= Qt.Key_9.valueOf()) || event.key() === Qt.Key_Period.valueOf()) {
+            // number or dot entered:
+            // give focus to first input widget in options tool bar and set text to number entered:
+            var w = OptionsToolBar.getFirstInputWidget();
+            if (!isNull(w)) {
+                w.setFocus(Qt.OtherFocusReason);
+                w.text = String.fromCharCode(event.key());
+                event.accept();
+                return;
+            }
         }
     }
 
@@ -2053,15 +2061,7 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
         selectable = false;
     }
 
-    var mod;
-    if (isFunction(event.getModifiers)) {
-        mod = event.getModifiers();
-    }
-    else {
-        mod = event.modifiers();
-    }
-
-    var altPressed = (mod & Qt.AltModifier)!==0;
+    var altPressed = isAltPressed(event);
     if (!altPressed || preview) {
         if (!isNull(action.idFromContextMenu)) {
             // user already chose an entity from the context menu:
@@ -2090,7 +2090,6 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
     var ret = RObject.INVALID_ID;
     var doc = di.getDocument();
     var menu = new QMenu(EAction.getGraphicsView());
-    var a, r;
     menu.objectName = "EntityContextMenu";
 
     // reacts to hovering and clicking of context menu items:
@@ -2112,6 +2111,7 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
     // invalid entity id is used to to cancel:
     entityIds.push(RObject.INVALID_ID);
 
+    var a, reactor;
     for (var i=0; i<entityIds.length && i<20; i++) {
         var id = entityIds[i];
         var str;
@@ -2143,16 +2143,18 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
             a.icon = icon;
         }
 
-        r = new Reactor(id);
-        a.triggered.connect(r, "trigger");
-        a.hovered.connect(r, "hover");
+        reactor = new Reactor(id);
+        a.triggered.connect(reactor, "trigger");
+        a.hovered.connect(reactor, "hover");
     }
 
     // show context menu:
     if (!menu.isEmpty()) {
         var prev = QCoreApplication.testAttribute(Qt.AA_DontShowIconsInMenus);
         QCoreApplication.setAttribute(Qt.AA_DontShowIconsInMenus, false);
+        this.waitingForContextMenu = true;
         menu.exec(new QPoint(QCursor.pos().x(), QCursor.pos().y()+10));
+        this.waitingForContextMenu = false;
         QCoreApplication.setAttribute(Qt.AA_DontShowIconsInMenus, prev);
         di.clearPreview();
     }
@@ -2203,6 +2205,10 @@ EAction.warnNotLineArcCircleEllipseSpline = function() {
     EAction.handleUserWarning(qsTr("Entity is not a line, arc, circle, ellipse or spline."));
 };
 
+EAction.warnNotLineArcCircleSplinePolyline = function() {
+    EAction.handleUserWarning(qsTr("Entity is not a line, arc, circle, spline or polyline."));
+};
+
 EAction.warnNotLineArcCircleEllipseSplinePolyline = function() {
     EAction.handleUserWarning(qsTr("Entity is not a line, arc, circle, ellipse, spline or polyline."));
 };
