@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -340,6 +340,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
 
     // no properties to show or 'No Selection' chosen:
     if (groups.length===0 /*|| selectionCombo.currentIndex===0*/) {
+        selectionCombo.clear();
         selectionCombo.insertItem(0, qsTr("No Selection"));
         layerCombo.clear();
         colorCombo.currentIndex = 0;
@@ -365,7 +366,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
 
         // grid layout with three columns and N rows for N property controls:
         gridLayoutGeometry = new QGridLayout(this.geometryGroup);
-        gridLayoutGeometry.setVerticalSpacing(4);
+        gridLayoutGeometry.setVerticalSpacing(2);
         gridLayoutGeometry.setColumnStretch(0,0);
         gridLayoutGeometry.setColumnStretch(1,1);
         gridLayoutGeometry.setColumnStretch(2,0);
@@ -380,7 +381,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
         // grid layout with four columns and N rows for N property controls:
         // (the fourth column is for the 'remove property' button)
         gridLayoutChild = new QGridLayout(this.childGroup);
-        gridLayoutChild.setVerticalSpacing(4);
+        gridLayoutChild.setVerticalSpacing(2);
         gridLayoutChild.setColumnStretch(0,0);
         gridLayoutChild.setColumnStretch(1,1);
         gridLayoutChild.setColumnStretch(2,0);
@@ -397,7 +398,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
             // grid layout with four columns and N rows for N property controls:
             // (the fourth column is for the 'remove property' button)
             gridLayoutCustom = new QGridLayout(this.customGroup);
-            gridLayoutCustom.setVerticalSpacing(4);
+            gridLayoutCustom.setVerticalSpacing(2);
             gridLayoutCustom.setColumnStretch(0,0);
             gridLayoutCustom.setColumnStretch(1,1);
             gridLayoutCustom.setColumnStretch(2,0);
@@ -437,7 +438,15 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
             //qDebug("value: ", value);
 
             var attributes = this.getPropertyAttributes(group, title);
-            var propertyTypeId = attributes.getPropertyTypeId();
+            //var propertyTypeId = attributes.getPropertyTypeId();
+            var propertyTypeId = RPropertyTypeId.getPropertyTypeId(group, title);
+
+            if (propertyTypeId.getId()===-1) {
+                // property is custom property
+                // make sure it knows it's titles, etc:
+                propertyTypeId.setCustomPropertyTitle(RSettings.getAppId());
+                propertyTypeId.setCustomPropertyName(title);
+            }
 
             //qDebug("isMixed: ", attributes.isMixed());
 
@@ -572,8 +581,18 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
 
                 var controls = undefined;
                 if (!isNull(gridLayout)) {
-                    // don't display any Z values:
-                    if (RSettings.getBoolValue("PropertyEditor/ShowZCoordinates", false)===false && title==="Z" && !propertyTypeId.isCustom()) {
+                    // don't display any Z values or polyline elevation:
+                    var hide = false;
+                    if (!propertyTypeId.isCustom()) {
+                        if (RSettings.getBoolValue("PropertyEditor/ShowZCoordinates", false)===false) {
+                            if (title===RPolylineEntity.PropertyVertexNZ.getPropertyTitle() ||
+                                title===RPolylineEntity.PropertyElevation.getPropertyTitle()) {
+                                hide = true;
+                            }
+                        }
+                    }
+
+                    if (hide) {
                         controls = undefined;
                     }
                     else {
@@ -586,7 +605,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
                     if (!isNull(controls) && !isNull(gridLayout)) {
                         row = gridLayout.rowCount();
                         var localTitle = title;
-                        if (propertyTypeId.isCustom() && group!==RSettings.getAppId()) {
+                        if (propertyTypeId.isCustom() && group!==RSettings.getAppId() && propertyTypeId.getCustomPropertyTitle()!=="Attributes") {
                             var pos = title.indexOf("_");
                             if (pos!==-1) {
                                 localTitle = title.mid(pos+1);
@@ -596,6 +615,11 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
                             localTitle = RSettings.translate("REntity", title);
                         }
 
+                        var l = attributes.getLabel();
+                        if (l.length>0) {
+                            localTitle = l;
+                        }
+
                         var label = new QLabel(localTitle + this.colon, groupBox);
                         label.textInteractionFlags = Qt.TextSelectableByMouse;
                         label.alignment = Qt.AlignRight | Qt.AlignVCenter;
@@ -613,7 +637,9 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
                         }
 
                         // 'remove custom property' button:
-                        if (propertyTypeId.isCustom() && !isNull(gridLayoutCustom) && group===RSettings.getAppId()) {
+                        if (propertyTypeId.isCustom() && !isNull(gridLayoutCustom) &&
+                            group===RSettings.getAppId() && !attributes.isUndeletable()) {
+
                             var removeCustomPropertyButton = new QToolButton(this.widget);
                             removeCustomPropertyButton.icon = new QIcon(this.basePath + "/RemoveCustomProperty.svg");
                             removeCustomPropertyButton.iconSize = new QSize(12,12);
@@ -703,7 +729,7 @@ PropertyEditorImpl.prototype.updateGui = function(onlyChanges) {
 
     // add custom property button:
     if (!onlyChanges) {
-        if (RSettings.isXDataEnabled()) {
+        if (RSettings.isXDataEnabled() && RSettings.getBoolValue("PropertyEditor/AddCustomProperties", true)!==false) {
             var addCustomPropertyButton = new QToolButton(this.widget);
             addCustomPropertyButton.icon = new QIcon(this.basePath + "/AddCustomProperty.svg");
             addCustomPropertyButton.iconSize = new QSize(12,12);
@@ -731,6 +757,12 @@ PropertyEditorImpl.prototype.initControls = function(propertyTypeId, onlyChanges
     var attributes = this.getPropertyAttributes(propertyTypeId);
     var controls;
 
+    // used for custom properties only:
+    // invisible flag of other properties is handled in RPropertyEditor:
+    if (attributes.isInvisible()) {
+        return undefined;
+    }
+
     var objectName = PropertyEditorImpl.getControlObjectName(propertyTypeId);
     if (isNull(control) && onlyChanges) {
         // find existing control:
@@ -741,6 +773,20 @@ PropertyEditorImpl.prototype.initControls = function(propertyTypeId, onlyChanges
         }
     }
 
+    // property available on request:
+    // add button to request all properties:
+    if (attributes.isOnRequest()) {
+        var requestAllButton = new QToolButton(this.geometryGroup);
+        requestAllButton.objectName = objectName + "_request";
+        requestAllButton.text = qsTr("Show");
+        requestAllButton.toolTip = qsTr("Show all properties");
+        requestAllButton.clicked.connect(this, "requestAllProperties");
+        requestAllButton.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Preferred);
+
+        return new Array(requestAllButton);
+    }
+
+
     // Layer:
 //    if (propertyTypeId.getId()===REntity.PropertyLayer.getId()) {
 //        WidgetFactory.initLayerCombo(control, EAction.getDocument());
@@ -751,20 +797,27 @@ PropertyEditorImpl.prototype.initControls = function(propertyTypeId, onlyChanges
     if (propertyTypeId.getId()===RTextEntity.PropertyHAlign.getId()) {
         controls = this.initChoiceControls(
                     objectName, propertyTypeId, onlyChanges, control, false, true);
-//                    [ qsTr("Left"), qsTr("Center"), qsTr("Right"),
-//                      qsTr("Aligned"), qsTr("Middle"), qsTr("Fit")],
-//                    [ RS.HAlignLeft, RS.HAlignCenter, RS.HAlignRight,
-//                      RS.HAlignAlign, RS.HAlignMid, RS.HAlignFit]);
     }
 
     // Vertical alignment: combo box:
     else if (propertyTypeId.getId()===RTextEntity.PropertyVAlign.getId()) {
         controls = this.initChoiceControls(
                     objectName, propertyTypeId, onlyChanges, control, false, true);
-//                    [qsTr("Top"), qsTr("Middle"), qsTr("Base"), qsTr("Bottom")],
-//                    [RS.VAlignTop, RS.VAlignMiddle, RS.VAlignBase, RS.VAlignBottom]);
     }
 
+    // Polyline orientation: combo box:
+    else if (propertyTypeId.getId()===RPolylineEntity.PropertyOrientation.getId()) {
+        controls = this.initChoiceControls(
+                    objectName, propertyTypeId, onlyChanges, control, false, true);
+    }
+
+    // Arc dimension arc symbol type: combo box:
+    else if (propertyTypeId.getId()===RDimArcLengthEntity.PropertyDimArcSymbolType.getId()) {
+        controls = this.initChoiceControls(
+                    objectName, propertyTypeId, onlyChanges, control, false, true);
+    }
+
+
     // Hatch pattern: combo box with hatch names:
     else if (propertyTypeId.getId()===RHatchEntity.PropertyPatternName.getId()) {
         var patternNames = undefined;
@@ -1099,6 +1152,12 @@ PropertyEditorImpl.prototype.initChoiceControls = function(
     else if (propertyTypeId.getId()===RTextEntity.PropertyVAlign.getId()) {
         WidgetFactory.initVAlignCombo(control);
     }
+    else if (propertyTypeId.getId()===RPolylineEntity.PropertyOrientation.getId()) {
+        WidgetFactory.initOrientationCombo(control);
+    }
+    else if (propertyTypeId.getId()===RDimArcLengthEntity.PropertyDimArcSymbolType.getId()) {
+        WidgetFactory.initArcSymbolTypeCombo(control);
+    }
     else if (isOfType(control, QComboBox) /*&& propertyTypeId.getId()!==REntity.PropertyLayer.getId()*/) {
         control.clear();
         if (isNull(choices)) {
@@ -1233,6 +1292,24 @@ PropertyEditorImpl.prototype.filterChanged = function() {
     this.updateFromDocument(doc, false, entityTypeFilter, true);
 };
 
+/**
+ * Called when the user requests all properties to be shown.
+ */
+PropertyEditorImpl.prototype.requestAllProperties = function() {
+    var doc = EAction.getDocument();
+    if (isNull(doc)) {
+        return;
+    }
+
+    var selectionCombo = this.widget.findChild("Selection");
+    var entityTypeFilter = selectionCombo.itemData(selectionCombo.currentIndex);
+    if (isNull(entityTypeFilter)) {
+        entityTypeFilter = RS.EntityAll;
+    }
+
+    this.updateFromDocument(doc, false, entityTypeFilter, true, true);
+};
+
 PropertyEditorImpl.prototype.getAdjustedPropertyValue = function(propertyTypeId) {
     // show handle as hex reprensentation (string):
     if (propertyTypeId.getId()===RObject.PropertyHandle.getId()) {
